﻿#region License
/*
SecondLanguage Gettext Library for .NET
Copyright 2013 James F. Bellinger <http://www.zer7.com>

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must
not claim that you wrote the original software. If you use this software
in a product, an acknowledgement in the product documentation would be
appreciated but is not required.

2. Altered source versions must be plainly marked as such, and must
not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source
distribution.
*/
#endregion

using System;
using NUnit.Framework;

namespace SecondLanguage.Tests
{
    [TestFixture]
    sealed class GettextPluralParserTest
    {
        [Test]
        public void ErrorChecking()
        {
            foreach (var pluralForms in new[]
                {
                    "",
                    "nplurals=3",
                    "plural=n==1 ? 0 : n%10>=2 && n%10<=4 && (n%100<10 || n%100>=20) ? 1 : 2",
                    "nplurals=3; plural=n==1 ? 0 : n%10>=2 && n%10<=4 && (n%100<10 || n%100>=20) ? 1 TEST : 2"
                })
            {
                Assert.Throws<FormatException>(() =>
                    {
                        int nplurals; GettextPluralConverterFunc valueToIndexFunc;
                        GettextPluralParser.Parse(pluralForms, out nplurals, out valueToIndexFunc);
                    });
            }
        }

        [Test]
        public void PluralForms()
        {
            // See http://www.gnu.org/software/gettext/manual/html_node/Plural-forms.html for the following plural form.
            // Its complexity should work out the Plural-Forms parser rather well...
            int nplurals; GettextPluralConverterFunc valueToIndexFunc;
            GettextPluralParser.Parse("nplurals=3; plural=n==1 ? 0 : n%10>=2 && n%10<=4 && (n%100<10 || n%100>=20) ? 1 : 2",
                                      out nplurals, out valueToIndexFunc);
            Assert.AreEqual(3, nplurals);
            Assert.AreEqual(0, valueToIndexFunc(1));
            Assert.AreEqual(1, valueToIndexFunc(2));
            Assert.AreEqual(2, valueToIndexFunc(21));
            Assert.AreEqual(2, valueToIndexFunc(14));
            Assert.AreEqual(1, valueToIndexFunc(24));
            Assert.AreEqual(2, valueToIndexFunc(15));
            Assert.AreEqual(1, valueToIndexFunc(132));
            Assert.AreEqual(2, valueToIndexFunc(112));
        }

    }
}
