﻿#region License
/*
SecondLanguage Gettext Library for .NET
Copyright 2013 James F. Bellinger <http://www.zer7.com>

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must
not claim that you wrote the original software. If you use this software
in a product, an acknowledgement in the product documentation would be
appreciated but is not required.

2. Altered source versions must be plainly marked as such, and must
not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source
distribution.
*/
#endregion

using System;
using NUnit.Framework;

namespace SecondLanguage.Tests
{
    [TestFixture]
    sealed class SpecialFormattersTest
    {
        [Test]
        public void Alignment()
        {
            TranslatorFormatCallback printf = SpecialFormatters.C;
            Assert.AreEqual("Test    ", printf("%-8s", "Test"));
            Assert.AreEqual("    Test", printf("%8s", "Test"));
            Assert.AreEqual("Test    ", printf("%*s", -8, "Test"));
            Assert.AreEqual("    Test", printf("%*s", 8, "Test"));
            Assert.AreEqual("Test    ", printf("%-*s", 8, "Test"));
            Assert.AreEqual("    Test", printf("%-*s", -8, "Test"));
        }

        [Test]
        public void Flags()
        {
            TranslatorFormatCallback printf = SpecialFormatters.C;
            Assert.AreEqual("000255", printf("%06d", 255));
        }

        [Test]
        public void Precision()
        {
            TranslatorFormatCallback printf = SpecialFormatters.C;
            Assert.AreEqual("Te", printf("%.2s", "Test"));
            Assert.AreEqual("Test", printf("%.5s", "Test"));
            Assert.AreEqual("3.14", printf("%.2f", 3.1415));
            Assert.AreEqual("3.1", printf("%.1f", 3.1415));
            Assert.AreEqual("   3.1", printf("%*.*f", 6, 1, 3.1415));
            Assert.AreEqual("3.1   ", printf("%*.*f", -6, 1, 3.1415));
            Assert.AreEqual("4.2", printf("%.6g", 4.2));
            Assert.AreEqual("4.200000", printf("%.6f", 4.2));
        }

        [Test]
        public void PositionalArguments()
        {
            TranslatorFormatCallback printf = SpecialFormatters.C;
            Assert.AreEqual("Hello World! 1 + 1 = 2", printf("Hello %s! 1 + %3$d = %2$g", "World", 2, 1));
        }

        [Test]
        public void Type()
        {
            TranslatorFormatCallback printf = SpecialFormatters.C;
            Assert.AreEqual("7FFF", printf("%X", 32767));
            Assert.AreEqual("ff", printf("%x", 255));
        }
    }
}
